SystemOrganization addCategory: #'Comet-Core'!

WAObject subclass: #CTHandler
	instanceVariableNames: 'id pusher session response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTHandler class methodsFor: 'instance creation' stamp: 'lr 3/5/2009 21:47'!
on: aPusher
	^ self basicNew initializeOn: aPusher! !

!CTHandler methodsFor: 'private' stamp: 'lr 3/17/2009 20:26'!
basicConnect: aRequestContext
	session := aRequestContext session.
	response := aRequestContext response.
	response
		doNotCache;
		contentType: aRequestContext application contentType;
		nextPutAll: (self preambleFor: aRequestContext);
		nextPutAll: (String new: 1024 withAll: Character space)! !

!CTHandler methodsFor: 'private' stamp: 'lr 3/5/2009 21:36'!
basicDisconnect
	response ifNil: [ ^ self ].
	response destroy.
	response := nil! !

!CTHandler methodsFor: 'private' stamp: 'lr 10/28/2007 14:45'!
basicTerminate
	thisContext swapSender: nil.
	Processor activeProcess terminate! !

!CTHandler methodsFor: 'actions' stamp: 'lr 3/5/2009 21:38'!
connect: aRequestContext
	self
		basicDisconnect;
		basicConnect: aRequestContext.
	self pusher register: self.
	self basicTerminate! !

!CTHandler methodsFor: 'accessing' stamp: 'lr 7/12/2006 11:22'!
id
	^ id! !

!CTHandler methodsFor: 'initialization' stamp: 'jf 3/13/2009 17:57'!
initializeOn: aPusher
	self initialize.
	id := WAKeyGenerator current 
		keyOfLength: 16.
	pusher := aPusher! !

!CTHandler methodsFor: 'private' stamp: 'lr 3/5/2009 21:48'!
preambleFor: aRequestContext
	^ '<!!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"><html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en"><head><title>Comet</title></head><body>' , (String new: 1024 withAll: Character space)! !

!CTHandler methodsFor: 'actions' stamp: 'lr 3/25/2009 13:46'!
push: aString
	"Push aString to the receiver. Answer true, if the connection is still valid."

	response isNil
		ifTrue: [ ^ false ].
	^ [ response nextPutAll: aString; flushComet ]
		on: Error do: [ :err | ^ false ]! !

!CTHandler methodsFor: 'accessing' stamp: 'lr 7/12/2006 14:24'!
pusher
	^ pusher! !

!CTHandler methodsFor: 'accessing' stamp: 'lr 3/17/2009 20:26'!
session
	^ session! !

WAObject subclass: #CTPusher
	instanceVariableNames: 'mutex handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

CTPusher class
	instanceVariableNames: 'process delay'!

CTPusher class
	instanceVariableNames: 'process delay'!

!CTPusher class methodsFor: 'initialization' stamp: 'jf 1/22/2009 02:43'!
initialize
	WAPlatform current
		addToStartUpList: self;
		addToShutDownList: self! !

!CTPusher class methodsFor: 'private' stamp: 'lr 8/30/2006 10:25'!
pingProcess
	self allInstancesDo: [ :each | each ping ].
	delay wait! !

!CTPusher class methodsFor: 'initialization' stamp: 'jf 1/21/2009 17:42'!
shutDown
	process ifNil: [ ^ self ].
	WAPlatform current terminateProcess: process.
	process := nil! !

!CTPusher class methodsFor: 'initialization' stamp: 'MAD 2/20/2009 23:22'!
startUp
	self shutDown.
	delay := Delay forSeconds: 15.
	process := WAPlatform current
		forkCometProcessDo: [ [ self pingProcess ] repeat ]! !

!CTPusher class methodsFor: 'initialization' stamp: 'jf 1/22/2009 02:43'!
unload
	WAPlatform current
		removeFromStartUpList: self;
		removeFromShutDownList: self.
	self shutDown! !

!CTPusher methodsFor: 'accessing' stamp: 'jf 7/19/2009 18:23'!
canvas
	"Answer a fake rendering canvas for this pusher."
	
	^ WARenderer default context: WARenderContext new! !

!CTPusher methodsFor: 'accessing' stamp: 'lr 7/12/2006 11:50'!
handlers
	"Answer the current list of handlers."

	^ handlers! !

!CTPusher methodsFor: 'initialization' stamp: 'jf 1/22/2009 02:43'!
initialize
	super initialize.
	mutex := WAPlatform current semaphoreClass forMutualExclusion.
	handlers := Array new! !

!CTPusher methodsFor: 'public' stamp: 'jf 7/19/2009 17:43'!
javascript: aBlock
	"Evaluate aBlock with an SUScript instance and pushes the resulting script-string aString to all associated handlers."
	
	| script |
	script := (JSScript context: self renderContext)
				rendererClass: self rendererClass;
				yourself.
	aBlock value: script.
	self push: (String streamContents: [ :stream |
		stream
			nextPutAll: '<script type="text/javascript">parent.Comet.eval(';
			javascript: script asJavascript;
			nextPutAll: ')</script>' ])! !

!CTPusher methodsFor: 'accessing' stamp: 'lr 7/12/2006 15:01'!
mutex
	"Answer the mutex in use with the reciever."

	^ mutex! !

!CTPusher methodsFor: 'protected' stamp: 'lr 3/25/2009 13:47'!
ping
	"Ping all the associated handlers."

	self push: ' '! !

!CTPusher methodsFor: 'protected' stamp: 'lr 3/25/2009 13:47'!
push: aString
	"Push aString to all associated handlers."

	self mutex critical: [ handlers := handlers select: [ :each | each push: aString ] ]! !

!CTPusher methodsFor: 'protected' stamp: 'lr 8/30/2006 10:43'!
register: aHandler
	"Register a new handler into this pusher."

	mutex critical: [ handlers := handlers copyWith: aHandler ]! !

JSObject subclass: #CTComet
	instanceVariableNames: 'pusher url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTComet class methodsFor: 'configuration' stamp: 'lr 7/12/2006 12:10'!
headerField
	^ '_c'! !

!CTComet class methodsFor: 'configuration' stamp: 'lr 7/16/2009 08:54'!
terminator
	^ Character codePoint: 0! !

!CTComet methodsFor: 'actions' stamp: 'lr 7/3/2006 13:54'!
connect
	self call: 'connect'! !

!CTComet methodsFor: 'private' stamp: 'jf 7/19/2009 16:41'!
defaultUrl
	^ self renderContext actionUrl
		addParameter: self class headerField
		value: self pusher id! !

!CTComet methodsFor: 'actions' stamp: 'lr 7/3/2006 13:54'!
disconnect
	self call: 'disconnect'! !

!CTComet methodsFor: 'accessing-readonly' stamp: 'jf 7/19/2009 14:19'!
filter
	^ self session filters
		detect: [ :each | each isCometFilter ]
		ifNone: [ self session addFilterFirst: CTFilter new ]! !

!CTComet methodsFor: 'printing' stamp: 'lr 3/16/2009 19:22'!
printContentOn: aStream
	aStream nextPutAll: 'new Comet('; javascript: self url; nextPut: $)! !

!CTComet methodsFor: 'accessing' stamp: 'lr 7/3/2006 13:44'!
pusher
	^ pusher! !

!CTComet methodsFor: 'accessing' stamp: 'jf 7/19/2009 16:41'!
pusher: aPusher
	self url: (self renderContext actionUrl
		withParameter: self class headerField
		value: (self filter
			registerPusher: (pusher := aPusher)
			context: self requestContext))! !

!CTComet methodsFor: 'accessing' stamp: 'lr 7/3/2006 13:33'!
url
	^ url ifNil: [ url := self defaultUrl ]! !

!CTComet methodsFor: 'accessing' stamp: 'lr 7/3/2006 13:34'!
url: anUrl
	url := anUrl! !

!WAPackage class methodsFor: '*comet-core' stamp: 'lr 3/16/2009 19:28'!
cometCore
	^ self new
		name: 'Comet-Core';
		description: 'Server push, stream events from the server to client.';
		addDependency: 'Javascript-Core';
		addDependency: 'Seaside-Canvas';
		yourself! !

WAFileLibrary subclass: #CTLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTLibrary methodsFor: 'scripts' stamp: 'lr 3/16/2009 19:08'!
cometJs
	^ '/* Seaside Comet API
 * (c) 2006-2009 Lukas Renggli (renggli@gmail.com)
 */

Comet = function(url, options) {
	this.url = url;
	this.iframe = this.createIframe();
	document.body.appendChild(this.iframe);
};

Comet.prototype.connect = function() {
	this.iframe.onLoad = this.onLoad;
	this.iframe.src = this.url;
};

Comet.prototype.disconnect = function() {
	this.iframe.onLoad = this.iframe.src = null;
};

Comet.prototype.onLoad = function() {
	this.disconnect();
	this.connect();
};

Comet.prototype.createIframe = function() {
	var frame = document.createElement("iframe");
	frame.style.display = "block";
	frame.style.visibility = frame.style.overflow = "hidden";
	frame.style.width = frame.style.height = frame.style.border = frame.style.margin = frame.style.padding = "0";
	return frame;
};

Comet.eval = function(code) {
	return eval(code);
};'! !

!CTLibrary methodsFor: 'accessing' stamp: 'lr 1/2/2008 20:11'!
selectorsToInclude
	^ #( cometJs )! !

!WAPlatform methodsFor: '*comet-core' stamp: 'MAD 2/20/2009 23:24'!
forkCometProcessDo: aBlock
	self subclassResponsibility! !

WARequestFilter subclass: #CTFilter
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTFilter methodsFor: 'processing' stamp: 'lr 3/5/2009 21:37'!
cometForContext: aRequestContext
	^ (handlers
		at: (aRequestContext request fields
			at: CTComet headerField
			ifAbsent: [ ^ nil ])
		ifAbsent: [ ^ nil ])
			connect: aRequestContext! !

!CTFilter methodsFor: 'processing' stamp: 'lr 3/5/2009 21:23'!
handleFiltered: aRequestContext
	(aRequestContext response isKindOf: WAStreamedResponse)
		ifFalse: [ self error: 'Streaming-server required.' ].
	(aRequestContext request includesKey: CTComet headerField)
		ifTrue: [ self cometForContext: aRequestContext ]
		ifFalse: [ super handleFiltered: aRequestContext ]! !

!CTFilter methodsFor: 'initialization' stamp: 'lr 11/3/2008 20:43'!
initialize
	super initialize.
	handlers := Dictionary new! !

!CTFilter methodsFor: 'testing' stamp: 'lr 11/3/2008 20:50'!
isCometFilter
	^ true! !

!CTFilter methodsFor: 'public' stamp: 'lr 3/5/2009 21:47'!
registerPusher: aPusher context: aContext
	| handler |
	handler := CTHandler on: aPusher.
	handlers at: handler id put: handler.
	^ handler id! !

!WARequestFilter methodsFor: '*comet-core' stamp: 'lr 11/3/2008 20:50'!
isCometFilter
	^ false! !

!WAStreamedResponse methodsFor: '*comet-core' stamp: 'lr 3/25/2009 13:45'!
flushComet
	^ externalStream flushComet! !

!WACodecStream methodsFor: '*comet-core' stamp: 'lr 3/25/2009 13:45'!
flushComet
	^ stream flushComet! !

!WARenderer methodsFor: '*comet-core' stamp: 'jf 7/19/2009 18:43'!
comet
	^ (CTComet context: self context)
		rendererClass: self species;
		yourself! !
CTPusher initialize!
